@@ -3,12 +3,14 @@ import os
 import binascii
 import codecs
 import grpc
+from google.protobuf.json_format import MessageToJson
 from python_lnd_grpc.lnd_defaults import defaultTLSCertFilename, defaultAdminMacFilename, defaultNetwork, defaultRPCHost, defaultRPCPort
 import python_lnd_grpc.protos.rpc_pb2 as lnrpc
 import python_lnd_grpc.protos.rpc_pb2_grpc as rpcstub
 import python_lnd_grpc.protos.walletunlocker_pb2 as walletunlocker
 import python_lnd_grpc.protos.walletunlocker_pb2_grpc as walletunlockerstub
 
+
 # TODO: path for for system other than linux
 TLS_FILE_PATH = '~/.lnd/'
 ADMIN_MACAROON_PATH = '~/.lnd/data/chain/bitcoin/'
@@ -23,12 +25,24 @@ class Connector(object):
         macaroon_filename: str = defaultAdminMacFilename,
         network: str = defaultNetwork,
         grpc_host: str = defaultRPCHost,
-        grpc_port: str = defaultRPCPort):
+        grpc_port: str = defaultRPCPort,
+        custom_tls_path_full: str = None,
+        custom_macaroon_path_full: str = None,
+        output: str = None):
 
-        self.tls_file = os.path.expanduser(tls_file_path + tls_file_name)
-        print(self.tls_file)
-        self.full_macaroon_path = os.path.expanduser((macaroon_path + '{}/' + macaroon_filename).format(network))
-        print(self.full_macaroon_path)
+        # TODO move to separate method
+        if custom_tls_path_full is not None:
+            self.tls_file = os.path.expanduser(custom_tls_path_full)
+        else:
+            self.tls_file = os.path.expanduser(tls_file_path + tls_file_name)
+        # print(self.tls_file)
+
+        # TODO move to separate method
+        if custom_macaroon_path_full is not None:
+            self.full_macaroon_path = os.path.expanduser(custom_macaroon_path_full)
+        else:
+            self.full_macaroon_path = os.path.expanduser((macaroon_path + '{}/' + macaroon_filename).format(network))
+        # print(self.full_macaroon_path)
         with open(self.tls_file, 'rb') as f:
             self.cert = f.read()
         with open(self.full_macaroon_path, 'rb') as f:
@@ -37,6 +51,7 @@ class Connector(object):
         self.headers = {'Grpc-Metadata-macaroon': self.macaroon}
         self.host = grpc_host
         self.port = grpc_port
+        self.output = output
 
     @property
     def _ln_stub(self):
@@ -61,10 +76,18 @@ class Connector(object):
     def auth_metadata_plugin(self, _, callback):
         callback([("macaroon", self.macaroon)], None)
 
+    def out(self, response):
+        if self.output == "json":
+            return MessageToJson(response)
+        elif self.output == "json-none-included":
+            return MessageToJson(response, including_default_value_fields=True)
+        else:
+            return response
+
 
 class WalletUnlocker(Connector):
     # https://api.lightning.community/#changepassword
-    def change_password(self, current_password: bytes, new_password: bytes):
+    def changepassword(self, current_password: bytes, new_password: bytes):
         request = walletunlocker.ChangePasswordRequest(
             current_password=current_password,
             new_password=new_password,
@@ -73,7 +96,7 @@ class WalletUnlocker(Connector):
         return response
 
     # https://api.lightning.community/#genseed
-    def gen_seed(self, aezeed_passphrase: bytes, seed_entropy: bytes):
+    def genseed(self, aezeed_passphrase: bytes, seed_entropy: bytes):
         request = walletunlocker.GenSeedRequest(
             aezeed_passphrase=aezeed_passphrase,
             seed_entropy=seed_entropy,
@@ -82,7 +105,7 @@ class WalletUnlocker(Connector):
         return response
 
     # https://api.lightning.community/#initwallet
-    def init_wallet(
+    def initwallet(
         self, 
         wallet_password: bytes,
         cipher_seed_mnemonic,
@@ -100,7 +123,7 @@ class WalletUnlocker(Connector):
         return response
 
     # https://api.lightning.community/#unlockwallet
-    def unlock_wallet(self, wallet_password: bytes, recovery_window: int, channel_backups):
+    def unlockwallet(self, wallet_password: bytes, recovery_window: int, channel_backups):
         request = walletunlocker.UnlockWalletRequest(
             wallet_password=wallet_password,
             recovery_window=recovery_window,
@@ -112,15 +135,15 @@ class WalletUnlocker(Connector):
 
 class LNDMethods(Connector):
     # https://api.lightning.community/#abandonchannel
-    def abandon_channel(self, channel_point):
+    def abandonchannel(self, channel_point):
         request = lnrpc.AbandonChannelRequest(
             channel_point = channel_point,
         )
         response = self._ln_stub.AbandonChannel(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#closechannel
-    def close_channel(self, channel_point, force: bool, target_conf: int, sat_per_byte: int, delivery_address: str):
+    def closechannel(self, channel_point, force: bool, target_conf: int, sat_per_byte: int, delivery_address: str):
         request = lnrpc.CloseChannelRequest(
             channel_point=channel_point,
             force=force,
@@ -129,20 +152,20 @@ class LNDMethods(Connector):
             delivery_address=delivery_address,
         )
         response = self._ln_stub.CloseChannel(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#channelbalance
-    def channel_balance(self):
+    def channelbalance(self):
         response = self._ln_stub.ChannelBalance(lnrpc.ChannelBalanceRequest())
-        return response
+        return self.out(response)
     
     # https://api.lightning.community/#connectpeer
-    def connect_peer(self, pubkey: str, host: str, perm: bool = False):
+    def connectpeer(self, pubkey: str, host: str, perm: bool = False):
         #async when perm = True
         address = lnrpc.LightningAddress(pubkey=pubkey, host=host)
         request = lnrpc.ConnectPeerRequest(addr = address, perm = perm)
         response = self._ln_stub.ConnectPeer(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#decodepayreq
     def decode_pay_req(self, pub_key: str):
@@ -150,81 +173,90 @@ class LNDMethods(Connector):
             pub_key=pub_key,
         )
         response = self._ln_stub.DecodePayReq(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#describegraph
-    def describe_graph(self, include_unannounced: bool):
+    def describegraph(self, include_unannounced: bool):
+        # TODO define specific variables for big message channel, format length of the line
+        channel = grpc.secure_channel((self.host + ":" + self.port), grpc.ssl_channel_credentials(self.cert), options=[('grpc.max_send_message_length', 20000000), ('grpc.max_receive_message_length', 20000000)])
+        stub = rpcstub.LightningStub(channel)
         request = lnrpc.ChannelGraphRequest(
             include_unannounced=include_unannounced,
         )
-        response = self._ln_stub.DescribeGraph(request)
-        return response
+        response = stub.DescribeGraph(request, metadata=[('macaroon', self.macaroon)])
+        return self.out(response)
 
     # https://api.lightning.community/#disconnectpeer
-    def disconnect_peer(self, pub_key: str):
+    def disconnect(self, pub_key: str):
         request = lnrpc.DisconnectPeerRequest(
             pub_key=pub_key,
         )
         response = self._ln_stub.DisconnectPeer(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#estimatefee
-    def estimate_fee(self, addr_to_amount, target_conf: int):
+    def estimatefee(self, addr_to_amount, target_conf: int):
         request = lnrpc.EstimateFeeRequest(
             AddrToAmount=addr_to_amount,
             target_conf=target_conf,
         )
         response = self._ln_stub.EstimateFee(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#feereport
-    def fee_report(self):
+    def feereport(self):
         response = self._ln_stub.FeeReport(lnrpc.FeeReportRequest())
-        return response
+        return self.out(response)
 
      # https://api.lightning.community/#getchaninfo
-    def get_chan_info(self, chan_id: int):
+    def getchaninfo(self, chan_id: int):
         request = lnrpc.ChanInfoRequest(
             chan_id=chan_id,
         )
         response = self._ln_stub.GetChanInfo(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#getinfo
-    def get_info(self):
+    def getinfo(self):
         response = self._ln_stub.GetInfo(lnrpc.GetInfoRequest())
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#getnetworkinfo
-    def get_network_info(self):
+    def getnetworkinfo(self):
         response = self._ln_stub.GetNetworkInfo(lnrpc.NetworkInfoRequest())
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#getnodeinfo
-    def get_node_info(self, pub_key: str, include_channels: bool):
+    def getnodeinfo(self, pub_key: str, include_channels: bool):
         request = lnrpc.NodeInfoRequest(
             pub_key=pub_key,
             include_channels=include_channels,
         )
         response = self._ln_stub.GetNodeInfo(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#getrecoveryinfo
-    def get_recovery_info(self):
+    def getrecoveryinfo(self):
         response = self._ln_stub.GetRecoveryInfo(lnrpc.GetRecoveryInfoRequest())
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#gettransactions
-    def get_transactions(self, start_height: int, end_height: int):
+    def gettransactions(self, start_height: int, end_height: int):
         request = lnrpc.GetTransactionsRequest(
             start_height=start_height,
             end_height=end_height,
         )
         response = self._ln_stub.GetTransactions(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#listchannels
-    def list_channels(self, active_only: bool, inactive_only: bool, public_only: bool, private_only: bool, peer: bytes):
+    def listchannels(
+        self, 
+        active_only: bool = None, 
+        inactive_only: bool = None, 
+        public_only: bool = None,
+        private_only: bool = None, 
+        peer: bytes = None):
         request = lnrpc.ListChannelsRequest(
             active_only=active_only,
             inactive_only=inactive_only,
@@ -233,10 +265,10 @@ class LNDMethods(Connector):
             peer=peer,
         )
         response = self._ln_stub.ListChannels(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#listinvoices
-    def list_invoices(self, pending_only: bool, index_offset: int, num_max_invoices: int, reverse: bool):
+    def listinvoices(self, pending_only: bool, index_offset: int, num_max_invoices: int, reverse: bool):
         request = lnrpc.ListInvoiceRequest(
             pending_only=pending_only,
             index_offset=index_offset,
@@ -244,10 +276,10 @@ class LNDMethods(Connector):
             reversed=reverse,
         )
         response = self._ln_stub.ListInvoices(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#listpayments
-    def list_payments(self, include_incomplete: bool, index_offset: int, max_payments: int, reverse: bool):
+    def listpayments(self, include_incomplete: bool, index_offset: int, max_payments: int, reverse: bool):
         request = lnrpc.ListPaymentsRequest(
             include_incomplete=include_incomplete,
             index_offset=index_offset,
@@ -255,41 +287,41 @@ class LNDMethods(Connector):
             reversed=reverse,
         )
         response = self._ln_stub.ListPayments(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#listpeers
-    def list_peers(self, latest_error: bool):
+    def listpeers(self, latest_error: bool = None):
         request = lnrpc.ListPeersRequest(
             latest_error=latest_error,
         )
         response = self._ln_stub.ListPeers(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#listunspent
-    def list_unspent(self, min_confs: int, max_confs: int):
+    def listunspent(self, min_confs: int, max_confs: int):
         request = lnrpc.ListUnspentRequest(
             min_confs=min_confs,
             max_confs=max_confs,
         )
         response = self.stub.ListUnspent(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#lookupinvoice
-    def lookup_invoice(self, r_hash_str: str, r_hash: bool):
+    def lookupinvoice(self, r_hash_str: str, r_hash: bool):
         request = lnrpc.PaymentHash(
             r_hash_str=r_hash_str,
             r_hash=r_hash,
         )
         response = self._ln_stub.LookupInvoice(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#newaddress
-    def new_address(self):
+    def newaddress(self):
         response = self._ln_stub.NewAddress(lnrpc.NewAddressRequest())
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#openchannel
-    def open_channel(
+    def openchannel(
         self, 
         node_pubkey: bytes,
         node_pubkey_string: str,
@@ -322,10 +354,10 @@ class LNDMethods(Connector):
             remote_max_value_in_flight_msat=remote_max_value_in_flight_msat,
         )
         response = self._ln_stub.OpenChannel(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#openchannelsync
-    def open_channel_sync(
+    def openchannelsync(
         self, 
         node_pubkey: bytes,
         node_pubkey_string: str,
@@ -358,24 +390,24 @@ class LNDMethods(Connector):
             remote_max_value_in_flight_msat=remote_max_value_in_flight_msat,
         )
         response = self._ln_stub.OpenChannelSync(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#pendingchannels
-    def pending_channels(self):
+    def pendingchannels(self):
         response = self._ln_stub.PendingChannels(lnrpc.PendingChannelsRequest())
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#restorechannelbackups
-    def restore_channel_backups(self, chan_backups, multi_chan_backup: bytes):
+    def restorechanbackup(self, chan_backups, multi_chan_backup: bytes):
         request = lnrpc.RestoreChanBackupRequest(
             chan_backups=chan_backups,
             multi_chan_backup=multi_chan_backup,
         )
         response = self._ln_stub.RestoreChannelBackups(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#sendcoins
-    def send_coins(self, addr: str, amount: int, target_conf: int, sat_per_byte: int, send_all: bool, label: str):
+    def sendcoins(self, addr: str, amount: int, target_conf: int, sat_per_byte: int, send_all: bool, label: str):
         request = lnrpc.SendCoinsRequest(
             addr=addr,
             amount=amount,
@@ -385,10 +417,10 @@ class LNDMethods(Connector):
             label=label,
         )
         response = self._ln_stub.SendCoins(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#sendmany
-    def send_many(self, AddrToAmount, target_conf: int, sat_per_byte: int, label: str):
+    def sendmany(self, AddrToAmount, target_conf: int, sat_per_byte: int, label: str):
         request = lnrpc.SendManyRequest(
             AddrToAmount=AddrToAmount,
             target_conf=target_conf,
@@ -396,10 +428,10 @@ class LNDMethods(Connector):
             label=label,
         )
         response = self._ln_stub.SendMany(request)
-        return response
+        return self.out(response)
 
     # https://api.lightning.community/#sendpaymentsync
-    def send_payments_sync(
+    def sendpaymentssync(
         self,
         dest: bytes,
         dest_string: str,
@@ -434,27 +466,27 @@ class LNDMethods(Connector):
             dest_features=dest_features,
         )
         response = self._ln_stub.SendPaymentSync(request)
-        return response
+        return self.out(response)
 
-    def send_to_route(self, payment_hash: bytes, payment_hash_string: str, route):
+    def sendtoroute(self, payment_hash: bytes, payment_hash_string: str, route):
         request = lnrpc.SendToRouteRequest(
             payment_hash=payment_hash,
             payment_hash_string=payment_hash_string,
             route=route,
         )
         response = self._ln_stub.SendToRouteSync(request)
-        return response
+        return self.out(response)
 
-    def subscribe_channel_balance(self):
+    def subscribechannelbackups(self):
         response = self._ln_stub.SubscribeChannelBackups(lnrpc.ChannelBackupSubscription())
-        return response
+        return self.out(response)
 
     
 
     # https://api.lightning.community/#walletbalance
-    def wallet_balance(self):
+    def walletbalance(self):
         response = self._ln_stub.WalletBalance(lnrpc.WalletBalanceRequest()) 
-        return response
+        return self.out(response)
 
     '''
     def forwarding_history(self):
